#include "Lighting.h"
#include <FastLED.h>

#define MAX_HEAT_INDEX 240.0
#define PWMRANGE 1023

DEFINE_GRADIENT_PALETTE(_sunrise_p) {
	0, 0, 0, 0,
	8, 0, 0, 16,
	128, 255, 128, 0,
	224, 255, 255, 0,
	240, 255, 255, 255, // Hack to get around LINEARBLEND problem (for now, hopefully)
	255, 255, 255, 255
};

Lighting::Lighting(NaturalLight* naturalLight, Weather* weather, float cloudCoverLimit) {
	_naturalLight = naturalLight;
	_weather = weather;
	_cloudCoverLimit = cloudCoverLimit;

	CRGB leds = CRGB(_ledCount);
	_leds = &leds;

	analogWriteFreq(PWMRANGE);
}

float Lighting::getIndexMultiplier(int now, int startTime, int endTime, bool swap) {
	float a = swap ? endTime - now : now - startTime;
	float b = endTime - startTime;
	return constrain(a / b, 0, 1);
}

float Lighting::getHeatIndex(int now) {
	return now >= _naturalLight->getSunset()
							? getIndexMultiplier(now, _naturalLight->getSunset(), _naturalLight->getAstronomicalTwilightEnd(), true)
							: getIndexMultiplier(now, _naturalLight->getAstronomicalTwilightBegin(), _naturalLight->getSunrise(), false);
}

float Lighting::getPaletteHeatIndex(int now){
	return MAX_HEAT_INDEX * getHeatIndex(now);
}

float Lighting::getBrightness() {
	float cloudCover = (float)_weather->getCloudCover() / 100.0;
	float multiplier = cloudCover * (_cloudCoverLimit / 100.0);
	// if (_weather->getCondition() == WeatherCondition::Other)
	// 	multiplier *= 2.0;
	return 1 - multiplier;
}

float Lighting::getPaletteBrightness(){
	return getBrightness() * 255.0;
}

float Lighting::byteToFraction(byte input){
	return input / 255.0;
}

void Lighting::outputColour(CRGB colour){
	if (_useStrip)
		FastLED.showColor(colour);

	if (_usePWM){
		_redLed->writeFraction(byteToFraction(colour.r));
		_greenLed->writeFraction(byteToFraction(colour.g));
		_blueLed->writeFraction(byteToFraction(colour.b));
	}
}

void Lighting::updateRGB(int now) {
	byte heatIndex = getPaletteHeatIndex(now);
	byte brightness = getPaletteBrightness();
	if (heatIndex != _lastHeatIndex || brightness!=_lastBrightness) {
		CRGB colour = ColorFromPalette(_sunrise, heatIndex, brightness, LINEARBLEND);

		if (_weather->getCondition() == WeatherCondition::Thunder && millis() >= _nextLightningFlash) {
			int flashes = random8(2, 8);
			for (int i = 0; i < flashes; i++) {
				outputColour(CRGB::White);
				delay(random8(10, 20));
				outputColour(colour);
				delay(random8(40, 80));
			}
			_nextLightningFlash = millis() + (random8(1, 60) * 1000);
		}

		outputColour(colour);
	}
	_lastHeatIndex = heatIndex;
	_lastBrightness = brightness;
}

void Lighting::updateWhite(){
	float brightness = (_heatIndex - 0.5) / 0.5;
	brightness = brightness * getBrightness();
	if (brightness < 0)
		brightness = 0;
	_whiteLed->writeFraction(brightness);
}

void Lighting::update(int now) {
	_heatIndex = getHeatIndex(now);
	_brightness = getBrightness();

	updateRGB(now);
	updateWhite();
}