#include "Config.h"
#include <FastLED.h>
#include <time.h>
#include "Networking.h"
#include "Sensors.h"
#include "NaturalLight.h"
#include "Lighting.h"
#include "Weather.h"

#define BASE_TOPIC "/home/sensors/" HOSTNAME
#define PH_TOPIC BASE_TOPIC "/ph"
#define TEMPERATURE_TOPIC BASE_TOPIC "/temperature"
#define LIGHT_INDEX_TOPIC BASE_TOPIC "/lightindex"
#define BRIGHTNESS_TOPIC BASE_TOPIC "/brightness"

/*----------------------------------------------------------+
| Need this here until I figure out why I can't instantiate |
| within the Sensors class...                               |
+----------------------------------------------------------*/
#include <OneWire.h>
#include <DallasTemperature.h>
#include <WiFiClient.h>

WiFiClient _wifiClient;
OneWire oneWire(TEMPERATURE_PIN);
DallasTemperature temperatureSensor(&oneWire);

LEDOutput _redLed = LEDOutput(R_PIN, true);
LEDOutput _greenLed = LEDOutput(G_PIN);
LEDOutput _blueLed = LEDOutput(B_PIN);
LEDOutput _whiteLed = LEDOutput(WHITE_PIN);

Networking _networking = Networking(HOSTNAME, NETWORK_NAME, PASSWORD, MQTT_SERVER);
Sensors _sensors = Sensors(TEMPERATURE_PIN, &_networking, &temperatureSensor);
NaturalLight _naturalLight = NaturalLight(_wifiClient, LATITUDE, LONGITUDE, TIMEZONE_OFFSET);
Weather _weather = Weather(_wifiClient, LATITUDE, LONGITUDE);
Lighting _lighting = Lighting(&_naturalLight, &_weather, CLOUD_COVER_LIMIT);

void setupTime(){
	configTime(0, 0, NTP_POOL);
	setenv("TZ", "GMT0BST,M3.5.0/2,M10.5.0/2", 0);
	delay(1000);
}

void setup() {
	Serial.begin(115200);
	analogWriteRange(PWMRANGE);

#ifdef LED_USE_STRIP
	_lighting.setupStrip<RGB_PIN, LED_COUNT, WHITE_PIN>();
#endif

#ifdef LED_USE_PWM
	_lighting.setupPWM<&_redLed, &_greenLed, &_blueLed, &_whiteLed>();
#endif

	_networking.setup();
	_sensors.setup();

	delay(2000);
	setupTime();

	_naturalLight.update();
	_weather.update();
}

void publishFloat(const char* topic, float value){
	char output[6];
	dtostrf(value, 0, 2, output);
	_networking.publish(topic, output, true);
}

void doLighting() {
	EVERY_N_HOURS(3){
		_naturalLight.update();
	}

	EVERY_N_MINUTES(2){
		_weather.update();
	}

	EVERY_N_SECONDS(1) {
		publishFloat(LIGHT_INDEX_TOPIC, _lighting.getCurrentHeatIndex());
		publishFloat(BRIGHTNESS_TOPIC, _lighting.getCurrentBrightness());
	}

	_lighting.update(time(nullptr));

	_redLed.loop();
	_blueLed.loop();
	_greenLed.loop();
	_whiteLed.loop();
}

void publishTemperature(){
	String temperature = _sensors.getTemperature();
	if (temperature != "Pending...")
		_networking.publish(TEMPERATURE_TOPIC, temperature.c_str(), true);
}

void publishReadings(){
	EVERY_N_SECONDS(2) {
		publishTemperature();
	}
}

void syncTime(){
	EVERY_N_SECONDS(3600) {
		configTime(0, 0, NTP_POOL);
	}
}

void loop() {
	_networking.loop();
	syncTime();
	doLighting();
	publishReadings();
	delay(10);
}